/* Funktionen zur Generation eindeutiger IDs für Arduinos
 * 
 * ID hat Wert zwischen 0 und 63 (6 Bit)
 * 
 * EEPROM-Belegung:
 * 0-6  Code zur eindeutigen Sicherstellung dass ID korrekt generiert wurde
 *   7  ID
 * 
 * Zum neuprogrammieren in Zeile 25 "cy = i*i+5" die Zahl 5 einmalig ändern
 * 
 * Zur Genrierung der "zufälligen ID" werden 8 Analogeingänge gelesen und addiert.
 * Alternativ kann einmalig eine feste ID programmiert werden.
 * 
 */

#include <EEPROM.h>

byte ID_getID()
{
  byte cx, cy, id;
  bool codeOK = true;
  int i;
  
  //Serial.println("Lese EEPROM-Code");
  for (i = 0; i < 7; i++)
  {
    cx = EEPROM.read(i);
    cy = i*i+5;   // 5, 7, 9, 14, 19, 30, 41 
    //Serial.print("Code Soll: "); Serial.print(cy); Serial.print(" Ist:"); Serial.println(cx);
    if (cx != cy) codeOK = false;
  }
  id = EEPROM.read(7);
  //Serial.print("ID: "); Serial.println(id);
  if (not codeOK)
    id = ID_generateID();
  Serial.print("Device ID 0x"); Serial.print(id,HEX); Serial.println(" starting...");
  return id;
}

byte ID_generateID()
{
  byte cx, cy, id;
  int rndSum = 0;
  int i;

  Serial.println("Loesche EEPROM und generiere ID");

  // Code schreiben
  for (i = 0 ; i < 7 ; i++) 
  {
    cy = i*i+5;
    EEPROM.write(i, cy);  
  }
  // ID Generieren
  rndSum += analogRead(A0);
  rndSum += analogRead(A1);
  rndSum += analogRead(A2);
  rndSum += analogRead(A3);
  rndSum += analogRead(A4);
  rndSum += analogRead(A5);
  rndSum += analogRead(A6);
  rndSum += analogRead(A7);
  id = rndSum%64;
  Serial.print("Sum of Analog Inputs: "); Serial.println(rndSum);
  Serial.print("ID: "); Serial.println(id, HEX);  
  EEPROM.write(7, id);      
  
  // Rest vom EEPROM löschen
  for (i = 8; i < EEPROM.length() ; i++)
  { 
    EEPROM.write(i, 0);      
  }
  Serial.println("EEPROM geloescht.");
  return id;
}

