/* MyCAN Testprogramm
 *  Nutzt ID_EEPROM zur Generation einer ID für den Arduino,
 *  Initialisiert den MyCAN-Bus, 
 *  und führt einen Buslast-Test durch. 
 *  Dazu sollten 2-3 Teilnehmer am MyCAN hängen.
 *  
 *  Es werden zufällige Nachrichten generiert, die eine
 *  mathematische Formel lösen. Die empfangenen Nachrichten
 *  werden dahingehend geprüft, ob die Formel noch stimmt.
 *  
 *  Die Statistiken per RS232 zeigen die Anzahl der korrekt
 *  Empfangenen Nachrichten sowie die Fehlerrate an.
 */

unsigned char deviceID;

void setup() 
{
  Serial.begin(9600);
  deviceID = ID_getID();
  MC_setup();
}

void loop() 
{
  int i;
  unsigned long A,B,C, message, cntMsgOK = 0, cntMsgNOK = 0;
  String line;
  float perc;

  // Sende 3 Testnachrichten
  for (int n = 0; n < 3; n++)
  {
    // test Message pattern:  A + B = C
    //                        7   9 = 10 Bits 26
    Serial.println("--------- Send ---------");
    for (i = 0; i < random(0,5); i++)
    {
      A = random(0,0x7F);
      B = random(0,0x1FF);
      C = A+B;
      B = (B<<7) + A;
      MC_sendMessage(C, B);
      // random delay
      delay(random(2000,5000));
    }
    //Serial.println("--------- Check Messages ---------");
    do
    {
      message = MC_getMessageFromRXqueue();
      if (message < 0xFFFFFFFF)
      {
        //MC_DebugLine("checking message: ",long(message),HEX);
        C = (message & 0x3FF0000) >> 16;
        B = (message & 0x000FF80) >> 7;
        A = (message & 0x000007F);
        line = MC_strLen(String(A),6) + "+" + MC_strLen(String(B),6) +"="+ MC_strLen(String(C),6) ;
        if (A+B == C) 
        {
          line += " OK"  ;
          cntMsgOK++;
        }
        else
        {
          line += " NOT OK!!!";
          Serial.println(line); // only print if not OK
          cntMsgNOK++;
        }
      }
    }
    while (message < 0xFFFFFFFF);
  }

  // Stats
  Serial.println("----------- Message Statistics -----------");
  perc = 100.0*cntMsgOK/(cntMsgOK+cntMsgNOK);
  line = "Messages OK:               "+MC_strLen(cntMsgOK, 9)+ " "+String(perc,1)+"%";
  Serial.println(line);
  perc = 100.0*cntMsgNOK/(cntMsgOK+cntMsgNOK);
  line = "Messages NOK:              "+MC_strLen(cntMsgNOK, 9)+ " "+String(perc,1)+"%";
  Serial.println(line);
  MC_printStats();
  delay(2000);
}
